// Sample code: Enroll templates into a collection, then run 1N identification

// This sample app demonstrates how to use 1N identification. First, a new collection is created.
// Next, templates are generated for several identities and enrolled into the collection.
// Finally, 1N identification is run to determine the identity of an anonymous template.
// To make the data enrolled in the collection persist, change the dbms configuration option
// Build on Linux: g++ -o identification --std=c++11 ./identification.cpp -I ../ -L ../trueface_sdk -l tf -fopenmp
// Build on macOS: g++ -o identification --std=c++11 ./identification.cpp -I ../ -L ../trueface_sdk -l tf

#include "trueface_sdk/tf_sdk.h"
#include <iostream>
#include <vector>
#include <string>

int main() {
    Trueface::ConfigurationOptions options;
    options.frModel = Trueface::FacialRecognitionModel::FULL;
    options.dbms = Trueface::DatabaseManagementSystem::NONE; // The data will not persist

    // If you want the data to persist, use this option.
//    options.dbms = Trueface::DatabaseManagementSystem::SQLITE;

    Trueface::SDK tfSdk(options);
    // TODO: replace the string with your license code.
    bool valid = tfSdk.setLicense("${LICENSE_CODE}");

    // Create a new collection or load existing collection
    const std::string databaseName = "myDatabase.db";
    const std::string collectionName = "myCollection";

    // If using Trueface::DatabaseManagementSystem::NONE this does not need to be called
    // but doing so is a harmless no-op
    auto retcode = tfSdk.createDatabaseConnection(databaseName);

    if (retcode != Trueface::ErrorCode::NO_ERROR) {
        std::cout << "Unable to create database connection\n";
        return -1;
    }

    retcode = tfSdk.createLoadCollection(collectionName);
    if (retcode != Trueface::ErrorCode::NO_ERROR) {
        std::cout << "Unable to create or load collection\n";
        return -1;
    }

    // Populate the collection with some identities
    std::vector<std::pair<std::string, std::string>> identitiesVec = {
            {"../images/armstrong_2.jpg", "Armstrong"},
            {"../images/armstrong_3.jpg", "Armstrong"}, // Can add the same identity more than once
            {"../images/aldrin.jpg", "Aldrin"}
    };

    for (const auto& identity: identitiesVec) {
        // Generate a template for each image
        retcode = tfSdk.setImage(identity.first);
        if (retcode != Trueface::ErrorCode::NO_ERROR) {
            std::cout << "Unable to set the image\n";
            return -1;
        }

        Trueface::Faceprint faceprint;
        retcode = tfSdk.getLargestFaceFeatureVector(faceprint);
        if (retcode != Trueface::ErrorCode::NO_ERROR) {
            std::cout << "Unable to generate template\n";
            return -1;
        }

        // Enroll the template into the collection
        std::string UUID;
        retcode = tfSdk.enrollTemplate(faceprint, identity.second, UUID);
        if (retcode != Trueface::ErrorCode::NO_ERROR) {
            std::cout << "Unable to enroll template\n";
            return -1;
        }
        // TODO: Can choose to store UUID for each faceprint which is enrolled into the collection
        // The UUID can later be used to delete the template from the collection
    }

    // Use image of armstrong as probe
    retcode = tfSdk.setImage("../images/armstrong_1.jpg");
    if (retcode != Trueface::ErrorCode::NO_ERROR) {
        std::cout << "Unable to set the image\n";
        return -1;
    }

    // Generate a template as the probe
    Trueface::Faceprint probeFaceprint;
    retcode = tfSdk.getLargestFaceFeatureVector(probeFaceprint);
    if (retcode != Trueface::ErrorCode::NO_ERROR) {
        std::cout << "Unable to generate template\n";
        return -1;
    }

    // Run identify function
    Trueface::Candidate candidate;
    bool found;
    const float threshold = 0.3;
    retcode = tfSdk.identifyTopCandidate(probeFaceprint, candidate, found, threshold);

    if (retcode != Trueface::ErrorCode::NO_ERROR) {
        std::cout << "There was an error with the call to identify" << std::endl;
        return -1;
    }

    if (!found) {
        std::cout << "Unable to find match" << std::endl;
        return -1;
    }

    std::cout << "Identity found: " << candidate.identity << std::endl;
    std::cout << "Match Similarity: " << candidate.similarityMeasure << std::endl;
    std::cout << "Match Probability: " << candidate.matchProbability << std::endl;

    // For the sake of the demonstration, remove all the templates from the collection
    for (const auto& identity: identitiesVec) {
        retcode = tfSdk.removeByIdentity(identity.second);

        if (retcode != Trueface::ErrorCode::NO_ERROR) {
            std::cout << "Unable to remove from collection\n";
            return -1;
        }
    }

    return 0;
}


