# Sample code: Get frame from webcam, run object detection on the frame, then display the frame
# Note: you will need to have the opencv-python module installed

import tfsdk
import cv2
import os

def draw_label(image, point, label,
               font=cv2.FONT_HERSHEY_SIMPLEX,
               font_scale=0.8, thickness=1):
    """Draw label on the image
    Args:
        img (str or binary): image path, base64 encoded image, numpy array or OpenCV image
        point (tuple): (x_label, y_label)
        label (str): The label you want to write
        font (int): your preferred font
        font_scale (float): scaling factor for the font
        thickness (int): thickness of text
    """
    size = cv2.getTextSize(label, font, font_scale, thickness)[0]
    x_label, y_label = point
    cv2.rectangle(
        image,
        (x_label, y_label - size[1]),
        (x_label + size[0], y_label),
        (255, 0, 255),
        cv2.FILLED)

    cv2.putText(
        image, label.capitalize(), point, font, font_scale,
        (0, 0, 0), thickness, cv2.LINE_AA)


options = tfsdk.ConfigurationOptions()
# Can set configuration options here
# ex:
# options.smallest_face_height = 40
# options.fd_mode = tfsdk.FACEDETECTIONMODE.VERSATILE
# options.fd_filter = tfsdk.FACEDETECTIONFILTER.BALANCED
# options.fr_model = tfsdk.FACIALRECOGNITIONMODEL.FULL

# Use the accurate object detector
options.obj_model = tfsdk.OBJECTDETECTIONMODEL.ACCURATE 

sdk = tfsdk.SDK(options)

# TODO: export your license token as TRUEFACE_TOKEN environment variable
is_valid = sdk.set_license(os.environ['TRUEFACE_TOKEN'])
if (is_valid == False):
	print("Invalid License Provided")
	print("Be sure to export your license token as TRUEFACE_TOKEN")
	quit()

# Use the default camera
cap = cv2.VideoCapture(0)

while(True):
	# Capture frame-by-frame
	_, frame = cap.read()

	# Set the image using the frame buffer. OpenCV stores images in BGR format
	res = sdk.set_image(frame, frame.shape[1], frame.shape[0], tfsdk.COLORCODE.bgr)
	if (res != tfsdk.ERRORCODE.NO_ERROR):
		print("Unable to set frame")
		continue

	# Run object detection
	bounding_boxes = sdk.detect_objects()
	if (len(bounding_boxes) > 0) :
		
		# Draw the bounding boxes and label on the frame
		for bounding_box in bounding_boxes:
			tl = (int(bounding_box.top_left.x), int(bounding_box.top_left.y))
			br = (int(bounding_box.top_left.x + bounding_box.width), int(bounding_box.top_left.y + bounding_box.height))

			# Draw the rectangle on the frame
			cv2.rectangle(frame, tl, br, (255, 0, 255), 3)

			label_string = sdk.get_object_label_string(bounding_box.label)
			draw_label(frame, tl, label_string)


    # Display the resulting frame
	cv2.imshow('frame', frame)
	if cv2.waitKey(1) & 0xFF == ord('q'):
		break

# When everything done, release the capture
cap.release()
cv2.destroyAllWindows()




